@@ -25,6 +25,7 @@ A driver for an efficient Tarantool/Box NoSQL in-memory storage.
             name          => "primary",   # self-descriptive space-id
             format        => "QqLlSsCc&", # pack()-compatible, Qq must be supported by perl itself, & stands for byte-string.
             default_index => 'idx1',
+            hashify       => [qw/ id f2 field3 f4 f5 f6 f7 f8 misc_string /], # turn each tuple into hash, field names according to format
         }, {
             #...
         } ],
@@ -49,7 +50,7 @@ A driver for an efficient Tarantool/Box NoSQL in-memory storage.
 use strict;
 use warnings;
 use Scalar::Util qw/looks_like_number/;
-use List::MoreUtils qw/each_arrayref/;
+use List::MoreUtils qw/each_arrayref zip/;
 use Time::HiRes qw/sleep/;
 
 use MR::IProto ();
@@ -141,6 +142,10 @@ C<pack()>-compatible tuple format string, allowed formats: C<QqLlSsCc&>,
 where C<&> stands for bytestring. C<Qq> usable only if perl supports
 int64 itself. Tuples' fields are packed/unpacked according to this C<format>.
 
+=item B<hashify> => $arg
+
+Override C<hashify> option (see below).
+
 =item B<indexes> => [ \%index, ... ]
 
 %index:
@@ -174,13 +179,28 @@ Must be set if there are more than one C<\%index>es.
 Space C<space> or C<name> to be used by default. Must be set if there are
 more than one C<\%space>s.
 
+=item B<hashify> => B<$coderef>
+
+Specify a callback to turn each tuple into a good-looking hash.
+It receives C<space> id and resultset as arguments. No return value needed.
+
+    $coderef = sub {
+        my ($space_id, $resultset) = @_;
+        $_ = { FieldName1 => $_->[0], FieldName2 => $_->[1], ... } for @$resultset;
+    };
+
+=item B<fields> => B<$arrayref>
+
+Specify an arrayref of fields names according to C<format> to turn each
+tuple into a good-looking hash.
+
 =item B<timeout> => $timeout_fractional_seconds_float || 23
 
 A common timeout for network operations.
 
 =item B<select_timeout> => $select_timeout_fractional_seconds_float || 2
 
-Select queries timeout for network operations. See L<select_retry>.
+Select queries timeout for network operations. See L</select_retry>.
 
 =item B<retry> => $retry_int || 1
 
@@ -208,6 +228,10 @@ example), and we B<can> try it again.
 This is also limited by C<retry>/C<select_retry>
 (depending on query type).
 
+=item B<retry_delay> => $retry_delay_fractional_seconds_float || 1
+
+Specify a delay between retries for network operations.
+
 =item B<raise> => $raise_bool || 1
 
 Should we raise an exceptions? If so, exceptions are raised when no more retries left and
@@ -241,7 +265,6 @@ sub new {
     $self->{raise}           = 1;
     $self->{raise}           = $arg->{raise} if exists $arg->{raise};
     $self->{hashify}         = $arg->{'hashify'} if exists $arg->{'hashify'};
-    $self->{default_raw}     = exists $arg->{default_raw} ? $arg->{default_raw} : !$self->{hashify};
     $self->{select_timeout}  = $arg->{select_timeout} || $self->{timeout};
     $self->{iprotoclass}     = $arg->{iprotoclass} || $class->IPROTOCLASS;
     $self->{_last_error}     = 0;
@@ -253,12 +276,12 @@ sub new {
         $ns = { %$ns };
         my $namespace = defined $ns->{space} ? $ns->{space} : $ns->{namespace};
         $ns->{space} = $ns->{namespace} = $namespace;
-        confess "ns[?] `space' not set" unless defined $namespace;
-        confess "ns[$namespace] already defined" if $namespaces{$namespace} or $ns->{name}&&$namespaces{$ns->{name}};
-        confess "ns[$namespace] no indexes defined" unless $ns->{indexes} && @{$ns->{indexes}};
+        confess "space[?] `space' not set" unless defined $namespace;
+        confess "space[$namespace] already defined" if $namespaces{$namespace} or $ns->{name}&&$namespaces{$ns->{name}};
+        confess "space[$namespace] no indexes defined" unless $ns->{indexes} && @{$ns->{indexes}};
         $namespaces{$namespace} = $ns;
         $namespaces{$ns->{name}} = $ns if $ns->{name};
-        confess "ns[$namespace] bad format `$ns->{format}'" if $ns->{format} =~ m/[^&lLsScCqQ ]/;
+        confess "space[$namespace] bad format `$ns->{format}'" if $ns->{format} =~ m/[^&lLsScCqQ ]/;
         $ns->{format} =~ s/\s+//g;
         my @f = split //, $ns->{format};
         $ns->{byfield_unpack_format} = [ map { /&/ ? 'w/a*' : "x$_" } @f ];
@@ -271,22 +294,26 @@ sub new {
         my $i = -1;
         for my $index (@{$ns->{indexes}}) {
             ++$i;
-            confess "ns[$namespace]index[($i)] no name given" unless length $index->{index_name};
+            confess "space[$namespace]index[($i)] no name given" unless length $index->{index_name};
             my $index_name = $index->{index_name};
-            confess "ns[$namespace]index[$index_name($i)] no indexes defined" unless $index->{keys} && @{$index->{keys}};
-            confess "ns[$namespace]index[$index_name($i)] already defined" if $inames->{$index_name} || $inames->{$i};
+            confess "space[$namespace]index[$index_name($i)] no indexes defined" unless $index->{keys} && @{$index->{keys}};
+            confess "space[$namespace]index[$index_name($i)] already defined" if $inames->{$index_name} || $inames->{$i};
             $index->{id} = $i unless defined $index->{id};
             $inames->{$i} = $inames->{$index_name} = $index;
-            int $_ == $_ and $_ >= 0 and $_ < @f or confess "ns[$namespace]index[$index_name] bad key `$_'" for @{$ns->{keys}};
+            int $_ == $_ and $_ >= 0 and $_ < @f or confess "space[$namespace]index[$index_name] bad key `$_'" for @{$ns->{keys}};
             $ns->{check_keys}->{$_} = int !! $ns->{string_keys}->{$_} for @{$index->{keys}};
             $index->{string_keys} ||= $ns->{string_keys};
         }
         if( @{$ns->{indexes}} > 1 ) {
-            confess "ns[$namespace] default_index not given" unless defined $ns->{default_index};
-            confess "ns[$namespace] default_index $ns->{default_index} does not exist" unless $inames->{$ns->{default_index}};
+            confess "space[$namespace] default_index not given" unless defined $ns->{default_index};
+            confess "space[$namespace] default_index $ns->{default_index} does not exist" unless $inames->{$ns->{default_index}};
         } else {
             $ns->{default_index} ||= 0;
         }
+        if($ns->{fields}) {
+            confess "space[$namespace] fields must be ARRAYREF" unless ref $ns->{fields} eq 'ARRAY';
+            confess "space[$namespace] fields number must match format" if @{$ns->{fields}} != @f;
+        }
     }
     $self->{namespaces} = \%namespaces;
     if (@{$arg->{spaces}} > 1) {
@@ -504,7 +531,7 @@ All of them have the same parameters:
 
 =item B<@tuple>
 
-A tuple to insert. All fields must be defined. All fields will be C<pack()>ed according to C<format> (see L<new>)
+A tuple to insert. All fields must be defined. All fields will be C<pack()>ed according to C<format> (see L</new>)
 
 =item B<%options>
 
@@ -696,8 +723,12 @@ sub _PackSelect {
 
 sub _PostSelect {
     my ($self, $r, $param) = @_;
-    if(!$param->{raw} && ref $param->{hashify} eq 'CODE') {
-        $param->{hashify}->($param->{namespace}->{namespace}, $r);
+    if(!$param->{raw}) {
+        if(ref $param->{hashify} eq 'CODE') {
+            $param->{hashify}->($param->{namespace}->{namespace}, $r);
+        } elsif( $param->{namespace}->{fields} ) {
+            $_ = { zip @{$param->{namespace}->{fields}}, @$_ } for @$r;
+        }
     }
 }
 
@@ -707,13 +738,17 @@ sub _PostSelect {
 
 Select tuple(s) from storage
 
-    my $tuple  = $box->Select($key)             or $box->Error && die $box->ErrorStr;
+    my $key = $id;
+    my $key = [ $firstname, $lastname ];
+    my @keys = ($key, ...);
+    
+    my $tuple  = $box->Select($key)              or $box->Error && die $box->ErrorStr;
     my $tuple  = $box->Select($key, \%options)   or $box->Error && die $box->ErrorStr;
     
-    my @tuples = $box->Select(@keys)            or $box->Error && die $box->ErrorStr;
+    my @tuples = $box->Select(@keys)             or $box->Error && die $box->ErrorStr;
     my @tuples = $box->Select(@keys, \%options)  or $box->Error && die $box->ErrorStr;
     
-    my $tuples = $box->Select(\@keys)           or die $box->ErrorStr;
+    my $tuples = $box->Select(\@keys)            or die $box->ErrorStr;
     my $tuples = $box->Select(\@keys, \%options) or die $box->ErrorStr;
 
 =over
@@ -738,9 +773,13 @@ in the storage
 
 =item *
 
-If you select C<< \@keys >> then C<< \@tuples >> will be returned upon success. @tuples will
+If you select C<< \@keys >> then C<< \@tuples >> will be returned upon success. C<< @tuples >> will
 be empty if there are no such keys, and false will be returned in case of error.
 
+=item *
+
+If you select using index on multiple fields each C<< $key >> should be gives as a key-tuple C<< $key = [ $key_field1, $key_field2, ... ] >>.
+
 =back
 
 =item B<%options>
@@ -755,20 +794,16 @@ Specify storage (by id or name) space to select from.
 
 Specify index (by id or name) to use.
 
-=item B<hashify> => $coderef
-
-Override C<hashify> option (see L<new>).
-
 =item B<raw> => $bool
 
-Don't C<hashify>.
+Don't C<hashify> (see L</new>).
 
 =item B<hash_by> => $by
 
 Return a hashref of the resultset. If you C<hashify> the result set,
 then C<$by> must be a field name of the hash you return,
-else it must be a number of field of the tuple.
-False will be returned in case of error.
+otherwise it must be a number of field of the tuple.
+C<False> will be returned in case of error.
 
 =back
 
@@ -811,7 +846,6 @@ sub Select {
         ) or return;
     }
 
-    $param->{raw} = $self->{default_raw} unless exists $param->{raw};
     $param->{want} ||= !1;
 
     $self->_PostSelect($r, { hashify => $param->{hashify}||$namespace->{hashify}||$self->{hashify}, %$param, namespace => $namespace });
@@ -849,7 +883,6 @@ sub SelectUnion {
     return [] unless @reqs;
     local $self->{raise} = $param->{raise} if defined $param->{raise};
     confess "bad param" if grep { ref $_ ne 'ARRAY' } @reqs;
-    $param->{raw} = $self->{default_raw} unless exists $param->{raw};
     $param->{want} ||= 0;
     for my $req (@reqs) {
         my ($param, $namespace) = $self->_validate_param($req, @select_param_ok);
@@ -1020,7 +1053,7 @@ Currently arithmetic operations are supported only for int32 (4-byte length) fie
 
 =item B<splice>, B<substr>
 
-Apply a perl-like L<splice> operation to C<< $field_num >>. B<$value> = [$OFFSET, $LENGTH, $REPLACE_WITH].
+Apply a perl-like L<perlfunc/splice> operation to C<< $field_num >>. B<$value> = [$OFFSET, $LENGTH, $REPLACE_WITH].
 substr is just an alias.
 
 =item B<append>, B<prepend>
